unit UniLabel;

{
   UniLabel.pas (C) 1999 David Mason

   This code is freeware:

        - It may be used by anyone, private or commercial.
        - It may be freely distributed without modification.

        - Do not modify and redistribute without checking
          with the author.
        - Do not remove this notice.
        - If you find it useful, please email me:
          drmason@xtra.co.nz
        - If this component is used in a program that
          makes you lots of money, please consider
          donating some to the author.

   THIS CODE IS PROVIDED AS IS, WITH NO GUARANTEES.
   USE THIS CODE AT YOUR OWN RISK. THE AUTHOR WILL
   NOT BE HELD RESPONSIBLE FOR ANY DAMAGE CAUSED BY
   THIS CODE.
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, UniDefs;

type
  TCustomUniLabel = class(TCustomLabel)
  protected
    FCodePageID:word;
    FCodePage:TCodePage;
    procedure DoDrawText(var Rect: TRect; Flags: Longint); override;
    procedure SetCodePage(value:TCodePage);
    property CodePage : TCodePage read FCodePage write SetCodePage;
  public
    constructor Create(AOwner:TComponent); override;
  end;

  TUniLabel = class(TCustomUniLabel)
  published
    property CodePage;
    property Align;
    property Alignment;
    property Anchors;
    property AutoSize;
    property Caption;
    property Color;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property FocusControl;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Transparent;
    property Layout;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

implementation

{ TCustomUniLabel }

constructor TCustomUniLabel.Create(AOwner: TComponent);
begin
  inherited;
  FCodePageID:=CodePageIdentifiers[cpANSI];
  FCodePage:=cpANSI; // Default to ANSI code page
end;

procedure TCustomUniLabel.DoDrawText(var Rect: TRect; Flags: Integer);
var text:string;
    textw:PWideChar;
    x,y:integer;
    l,TextWLength:word;
    Size:TSize;
begin
  Text := GetLabelText;
  if Length(text)>0 then begin

    if Font<>nil then begin
      // Convert the string to UNICODE
      TextW:=nil;
      ConvertToUnicode(FCodePageID,Text,TextW, l);
      TextWLength:=LStrLenW(TextW);

      Canvas.Font := Font;

      if (Flags and DT_CALCRECT)<>0 then begin
        // Probably called during autosizing,
        // caller just wants to know the size of the text
        Windows.GetTextExtentPointW(Canvas.Handle,TextW,TextWLength,Size);
        Rect.Right:=Size.cx;
        Rect.Bottom:=Size.cy;
      end else begin
        // Work out where to start writing
        y:=Rect.Top;
        x:=Rect.Left;
        if Alignment<>taLeftJustify then begin
          Windows.GetTextExtentPoint32W(Canvas.Handle,TextW,TextWLength,Size);
          case Alignment of
            taRightJustify: x:=Rect.Right-Rect.Left-Size.cx;
            taCenter:       x:=(Rect.Right-Rect.Left - Size.cx) div 2;
          end;
        end;

        // Write the text
        if not Enabled then
        begin
          Canvas.Font.Color := clBtnHighlight;
          TextOutW(Canvas.Handle, x+1,y+1,TextW,TextWLength);
          Canvas.Font.Color := clBtnShadow;
          TextOutW(Canvas.Handle, x,y,TextW,TextWLength);
        end else
          TextOutW(Canvas.Handle, x,y,TextW,TextWLength);
      end;

      // Clean up
      FreeMem(TextW,l);
    end;
  end;
end;


procedure TCustomUniLabel.SetCodePage(value: TCodePage);
begin
  FCodePageID:=CodePageIdentifiers[value];
  FCodePage:=value;
  Repaint;
end;

end.
